module SemverBump
  class SemVer

    MATCHER = /^(\d+)\.(\d+)\.(\d+)\.?([0-9A-Za-z]*)$/
    PARTS = [:major, :minor, :patch, :pre]
    attr_accessor *(PARTS << :pre_raw)

    def self.valid?(ver)
      MATCHER =~ ver
    end

    def initialize(ver)
      @major, @minor, @patch, @pre_raw = MATCHER.match(ver).captures.collect { |v|
        if v.to_i == 0 && v != '0'
          v
        else
          v.to_i
        end
      }
    end

    def pre
      @pre ||= get_pre
    end

    def get_pre
      return 0 unless pre_raw
      matches = (/(\d)/.match(pre_raw.to_s))
      return 0 unless matches.respond_to?(:captures)
      matches.captures.first.to_i
    end

    def pre_s
      "#{get_pre_s}#{pre}"
    end

    def get_pre_s
      @pre_s = pre_raw.to_s.gsub(/\d/,'')
      @pre_s = 'pre' if !@pre_s || @pre_s == ''
      @pre_s
    end

    (PARTS - [:pre,:minor]).each do |name|
      define_method(:"bump_#{name.to_s}") do
        send(:"#{name.to_s}=",(send(:"#{name.to_s}")+1))
        PARTS[PARTS.index(name)+1..4].each do |name|
          send(:"#{name.to_s}=",0)
        end
        self
      end
    end

    def bump_patch
      if pre && pre > 0
        send(:pre=,0)
      else
        send(:patch=,(send(:patch)+1))
      end
    end

    def bump_pre
      if !pre || pre == 0
        bump_patch
        send(:pre=,1)
      else
        send(:pre=,(send(:pre)+1))
      end
      self
    end

    def primary
      "#{@major}.#{@minor}.#{@patch}"
    end

    def to_s
      return [primary,pre_s].compact.join('.') if pre && pre > 0
      primary
    end

  end
end


